# do not look for pogfiles in this directory
# exclude_paths = ["dont_include"]  

# these files/directories shall be imported, even if they're out of path or if they're 
# include_paths = ["test_simple.py", "dont_include/test_anyway/"] 

# autoindex = False # change this to true to automatically search for pogfile.py in inferior directories.

@job(desc="Hello World job, this is the toplevel default target")
def top():
    print("Hello friend! from python")
    env.run("echo", "You can use env.run() to run a subprocess run similar to subprocess.run()")
    env.system("You can use env.system to try and use a system shell.")


@job("top", desc="This is a job that depends on the top job, dependencies are specified as strings")
def depends_on_top():
    pass # blank job, does nothing



# pogfile shared imports
import subprocess
import sys
import argparse
import os
import shutil

# /pogfile shared imports

import logging as lg

from . import core
from .core import job, JobManager
from .jobenv import JobEnv
from importlib import util
import importlib


def find_all_subpogs(root):
    return subpogs


def inner_importer(root, cli_args, _filename=None):
    """
    Runs the importer for a target file and
    directory,

    :return: g_jobs, a glob of all the jbos in that file
    """

    filename = _filename
    if _filename == None:
        filename = "pogfile"

    apath = os.path.abspath(os.path.join(root, filename))
    if not os.path.exists(apath):
        tapath = os.path.abspath(os.path.join(root, filename + ".py"))
        assert os.path.exists(
            tapath
        ), f" Neither {root + apath} or the .py version exists"
        apath = tapath

    orig_dir = os.path.dirname(apath)

    loader = importlib.machinery.SourceFileLoader("tmpPackage", apath)
    spec = importlib.util.spec_from_loader(loader.name, loader)
    mod = importlib.util.module_from_spec(spec)

    # pogfile extra symbols
    mod.job = job  # job decorator
    mod.cli_args = cli_args  # parsed command line arguments
    mod.pogmake_core = core  # Access to all pogmake
    # internals but not the importer or the frontend
    mod.orig_dir = orig_dir  # absolute path to origin of this pogfile
    # /pogfile extra symbols

    # standard imports
    mod.sys = sys
    mod.subprocess = subprocess
    mod.shutil = shutil
    mod.os = os
    mod.include_paths = []
    mod.exclude_paths = []
    mod.env = JobEnv(orig_dir, cli_args)
    
    if not hasattr(mod, "autoindex"):
    
        mod.autoindex = False

    loader.exec_module(mod)

    gjobs = mod.pogmake_core.get_gjobs()
    for path in mod.include_paths:
        jpath = os.path.join(root, path)

        if not os.path.exists(jpath):
            print(f"Include path in {apath} does not exist: {jpath}")
            continue

        if os.path.isdir(jpath):
            rjobs, modinfo = inner_importer(jpath, cli_args)
            gjobs.update(rjobs)

        else:
            rjobs, modinfo = inner_importer(
                os.path.dirname(jpath),
                cli_args,
                os.path.basename(jpath)
            )

            gjobs.update(rjobs)

    return gjobs, mod

def main_importer(root, cli_args, startfile=None):
    gjobs, startmod = inner_importer(root, cli_args, startfile)
    
    if startmod.autoindex:
        rjobs = autoindex_importer(root, cli_args)
        gjobs.update(rjobs)

    return gjobs


def autoindex_importer(root, cli_args, filename="pogfile"):
    subpogs = []
    first = True
    gjobs = {}
    excluded = []
    for r, d, f in os.walk(root):
        should_skip = False
        for path in excluded:
            if r.startswith(os.path.abspath(path) + os.sep) or r == path:
                should_skip = True

        if not should_skip:
            for fname in f:
                if fname == filename or fname == filename+".py":
                    rjobs, modinfo = inner_importer(r, cli_args)
                    gjobs.update(rjobs)
                    for epath in modinfo.exclude_paths:
                        excluded.append(os.path.abspath(os.path.join(r, epath)))

    return gjobs



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

    setuptools.setup(
        name="pogmake",  # Replace with your own username
        version="0.0.2",
        scripts=['pog', 'pogm', 'pog.bat'],
        author="Peter Li",
        author_email="peterlimail47@gmail.com",
        description="I needed a cross platform makefile",
        long_description=long_description,
        long_description_content_type="text/markdown",
        url="https://github.com/peterino/pmake",
        packages=setuptools.find_packages(),
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
        install_requires=[
            "markdown",
            "colorama",
        ],
        python_requires=">=3.6",
    )



import os
import subprocess
import logging as lg
from colorama import Fore

class JobEnv:
    """
    JobEnv class
    an 'env' object is created in each .py file's module scope. 

    And contains defaults with regards to file paths, arguments etc...
    
    It is reccomended that you use this JobEnv file in order to define/run 
    your jobs. Instead of os.system and subprocess.run(...).
    """
    def __init__(self, orig_dir, gargs):
        self.orig_dir = orig_dir 
        self.gargs = gargs
        self.verbosity_values = {
            "DEBUG" : 0,
            "INFO" : 1,
            "WARNING" : 2,
            "ERROR" : 3,
        }

    def dbg(self, ostr):
        """
        A debug print, this will only print if --verbose is passed into the program
        """
        self.log(ostr, "DEBUG")

    def log(self, ostr, verbosity=None):
        verbosity_str = verbosity
        verbo = self.verbosity_values[verbosity.upper()] if verbosity is not None else self.verbosity_values['INFO']
        set_verbo = self.verbosity_values[self.gargs.log_level]
        colors = [Fore.GREEN, "", Fore.YELLOW]

        if verbo >= set_verbo:
            print(f"{colors[verbo]}[{verbosity_str}]: {ostr}")
            
    def run(self, *run_args, cwd=None, **kwargs):
        """
        Run a job, analogous to subprocess.run, however defaults to the cwd
        that is the same as orig_dir
        """
        target_cwd = cwd
        if cwd is None:
            target_cwd = self.orig_dir
        
        subprocess.run(run_args, cwd=target_cwd)

    def system(self, systr, cwd=None):
        """
        Runs a system shell command, analogous to os.system but with a cwd
        wrapper, which will default to orig_dir
        """
        oldcwd = os.getcwd()
        
        target_cwd = cwd
        if cwd is None:
            target_cwd = self.orig_dir
            
        self.dbg(f"os.system >>> {systr} @ {cwd}")
                 
        os.chdir(target_cwd)
        os.system(systr)
        os.chdir(oldcwd)
